@@ -8,12 +8,9 @@ use warnings;
 use strict;
 
 use Fcntl ();
-use File::Spec ();
 
 use base 'App::Ack::Resource';
 
-use App::Ack;
-
 =head1 METHODS
 
 =head2 new( $filename )
@@ -53,16 +50,6 @@ sub name {
     return $_[0]->{filename};
 }
 
-sub basename {
-    my ( $self ) = @_;
-
-    # XXX definedness? pre-populate the slot with an undef?
-    unless ( exists $self->{basename} ) {
-        $self->{basename} = (File::Spec->splitpath($self->name))[2];
-    }
-
-    return $self->{basename};
-}
 
 =head2 $res->needs_line_scan( \%opts )
 
@@ -115,12 +102,12 @@ is true.
 sub reset {
     my $self = shift;
 
-    # Return if we haven't opened the file yet.
+    # return if we haven't opened the file yet
     if ( !defined($self->{fh}) ) {
         return;
     }
 
-    if ( !seek( $self->{fh}, 0, 0 ) && $App::Ack::report_bad_filenames ) {
+    if( !seek( $self->{fh}, 0, 0 ) && $App::Ack::report_bad_filenames ) {
         App::Ack::warn( "$self->{filename}: $!" );
     }
 
@@ -136,7 +123,7 @@ API: Close the resource.
 sub close {
     my $self = shift;
 
-    # Return if we haven't opened the file yet.
+    # return if we haven't opened the file yet
     if ( !defined($self->{fh}) ) {
         return;
     }
@@ -167,7 +154,7 @@ sub firstliney {
 
     my $fh = $self->open();
 
-    if ( !exists $self->{firstliney} ) {
+    unless(exists $self->{firstliney}) {
         my $buffer = '';
         my $rc     = sysread( $fh, $buffer, 250 );
         unless($rc) { # XXX handle this better?
@@ -188,7 +175,7 @@ sub open {
 
     return $self->{fh} if $self->{opened};
 
-    if ( ! open $self->{fh}, '<', $self->{filename} ) {
+    unless ( open $self->{fh}, '<', $self->{filename} ) {
         return;
     }
 
