@@ -1,24 +1,11 @@
 package App::Ack::Filter::Collection;
 
-=head1 NAME
-
-App::Ack::Filter::Collection
-
-=head1 DESCRIPTION
-
-The Ack::Filter::Collection class can contain filters and internally sort
-them into groups. The groups can then be optimized for faster filtering.
-
-Filters are grouped and replaced by a fast hash lookup. This leads to
-improved performance when many such filters are active, like when using
-the C<--known> command line option.
-
-=cut
-
 use strict;
 use warnings;
 use base 'App::Ack::Filter';
 
+use File::Spec 3.00 ();
+
 sub new {
     my ( $class ) = @_;
 
@@ -50,7 +37,17 @@ sub add {
     my ( $self, $filter ) = @_;
 
     if (exists $filter->{'groupname'}) {
-        my $group = ($self->{groups}->{$filter->{groupname}} ||= $filter->create_group());
+        my $groups = $self->{'groups'};
+        my $group_name = $filter->{'groupname'};
+
+        my $group;
+        if (exists $groups->{$group_name}) {
+            $group = $groups->{$group_name};
+        }
+        else {
+            $group = $groups->{$group_name} = $filter->create_group();
+        }
+
         $group->add($filter);
     }
     else {
