@@ -23,8 +23,6 @@ sub add {
     foreach my $ext (@{$extensions}) {
         $data->{lc $ext} = 1;
     }
-
-    return;
 }
 
 sub filter {
@@ -46,7 +44,9 @@ sub inspect {
 sub to_string {
     my ( $self ) = @_;
 
-    return join(' ', map { ".$_" } sort keys %{$self->{data}});
+    my $data = $self->{'data'};
+
+    return join(' ', map { ".$_" } (keys %$data));
 }
 
 1;
