@@ -18,15 +18,13 @@ sub add {
     my ( $self, $filter ) = @_;
 
     $self->{data}->{ $filter->{filename} } = 1;
-
-    return;
 }
 
 sub filter {
     my ( $self, $resource ) = @_;
 
     my $data = $self->{'data'};
-    my $base = $resource->basename;
+    my $base = (File::Spec->splitpath($resource->name))[2];
 
     return exists $data->{$base};
 }
