@@ -5,7 +5,6 @@ use warnings;
 use base 'App::Ack::Filter';
 
 use File::Spec 3.00;
-use App::Ack::Filter::MatchGroup ();
 
 sub new {
     my ( $class, $re ) = @_;
@@ -15,20 +14,16 @@ sub new {
 
     return bless {
         regex => $re,
-        groupname => 'MatchGroup',
     }, $class;
 }
 
-sub create_group {
-    return App::Ack::Filter::MatchGroup->new;
-}
-
 sub filter {
     my ( $self, $resource ) = @_;
 
-    my $re = $self->{'regex'};
+    my $re   = $self->{'regex'};
+    my $base = (File::Spec->splitpath($resource->name))[2];
 
-    return $resource->basename =~ /$re/;
+    return $base =~ /$re/;
 }
 
 sub inspect {
@@ -37,8 +32,6 @@ sub inspect {
     my $re = $self->{'regex'};
 
     print ref($self) . " - $re";
-
-    return;
 }
 
 sub to_string {
