@@ -1,43 +1,16 @@
 package App::Ack::Resources;
 
-=head1 NAME
-
-App::Ack::Resources
-
-=head1 SYNOPSIS
-
-A factory object for creating a stream of L<App::Ack::Resource> objects.
-
-=cut
-
 use App::Ack;
-use App::Ack::Resource;
 
-use File::Next 1.16;
-use Errno qw(EACCES);
+use File::Next 1.10;
 
 use warnings;
 use strict;
 
-sub _generate_error_handler {
-    my $opt = shift;
+=head1 SYNOPSIS
 
-    if ( $opt->{dont_report_bad_filenames} ) {
-        return sub {
-            my $msg = shift;
-            if ( $! == EACCES ) {
-                return;
-            }
-            App::Ack::warn( $msg );
-        };
-    }
-    else {
-        return sub {
-            my $msg = shift;
-            App::Ack::warn( $msg );
-        };
-    }
-}
+This is the base class for App::Ack::Resources, an iterator factory
+for App::Ack::Resource objects.
 
 =head1 METHODS
 
@@ -67,8 +40,7 @@ sub from_argv {
         File::Next::files( {
             file_filter     => $opt->{file_filter},
             descend_filter  => $descend_filter,
-            error_handler   => _generate_error_handler($opt),
-            warning_handler => sub {},
+            error_handler   => sub { my $msg = shift; App::Ack::warn( $msg ) },
             sort_files      => $opt->{sort_files},
             follow_symlinks => $opt->{follow},
         }, @{$start} );
@@ -90,8 +62,8 @@ sub from_file {
 
     my $iter =
         File::Next::from_file( {
-            error_handler   => _generate_error_handler($opt),
-            warning_handler => _generate_error_handler($opt),
+            error_handler   => sub { my $msg = shift; App::Ack::warn( $msg ) },
+            warning_handler => sub { my $msg = shift; App::Ack::warn( $msg ) },
             sort_files      => $opt->{sort_files},
         }, $file ) or return undef;
 
@@ -123,11 +95,9 @@ sub from_stdin {
 sub next {
     my $self = shift;
 
-    my $file = $self->{iter}->();
+    my $file = $self->{iter}->() or return;
 
-    return unless defined($file);
-
-    return App::Ack::Resource->new( $file );
+    return App::Ack::Resource::Basic->new( $file );
 }
 
 1;
