@@ -2,25 +2,20 @@ package FilterTest;
 
 use strict;
 use warnings;
-use parent 'Exporter';
+use base 'Exporter';
 
-use App::Ack::File;
-use File::Next;
+use App::Ack::Resource::Basic;
+use File::Find;
 use Util;
 use Test::More;
 
 our @EXPORT = qw(filter_test);
 
-sub swamp_files {
-    my @swamp_files;
+my @swamp_files;
 
-    my $files = File::Next::files( 't/swamp' );
-    while ( my $file = $files->() ) {
-        push( @swamp_files, $file );
-    }
-
-    return @swamp_files;
-}
+find(sub {
+    push @swamp_files, $File::Find::name if -f;
+}, 't/swamp');
 
 sub filter_test {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -41,8 +36,8 @@ sub filter_test {
         } grep {
             $filter->filter($_)
         } map {
-            App::Ack::File->new($_)
-        } swamp_files();
+            App::Ack::Resource::Basic->new($_)
+        } @swamp_files;
 
         sets_match(\@matches, $expected_matches, $msg);
     };
